@@ -254,6 +254,8 @@ class ContextProxy(metaclass=abc.ABCMeta):
         if self.protocol not in self.available_protocols(options=options):
             raise ValueError("Protocol %s is not available" % self.protocol)
 
+        self._hostname = hostname
+        self._service = service
         self.spn = None
         if service or hostname:
             self.spn = to_text("%s/%s" % (service if service else "HOST", hostname or "unspecified"))
@@ -272,11 +274,6 @@ class ContextProxy(metaclass=abc.ABCMeta):
         # Whether the context is wrapped inside another context - set by NegotiateProxy.
         self._is_wrapped = False
 
-        if options & NegotiateOptions.negotiate_kerberos and (
-            self.protocol == "negotiate" and "kerberos" not in self.available_protocols()
-        ):
-            raise FeatureMissingError(NegotiateOptions.negotiate_kerberos)
-
         if options & NegotiateOptions.wrapping_iov and not self.iov_available():
             raise FeatureMissingError(NegotiateOptions.wrapping_iov)
 
@@ -393,6 +390,19 @@ class ContextProxy(metaclass=abc.ABCMeta):
         """
         pass  # pragma: no cover
 
+    @abc.abstractmethod
+    def new_context(self) -> "ContextProxy":
+        """Creates a new security context.
+
+        Creates a new security context based on the current credential and
+        options of the current context. This is useful when needing to set up a
+        new security context without having to retrieve the credentials again.
+
+        Returns:
+            ContextProxy: The new security context.
+        """
+        pass  # pragma: no cover
+
     @abc.abstractmethod
     def step(self, in_token: typing.Optional[bytes] = None) -> typing.Optional[bytes]:
         """Performs a negotiation step.
